# --------------------------------------------------------------------------
# Log-Likelihood function for Salmon example
# --------------------------------------------------------------------------
sallik <- function(pars,y){
  # This function returns the negative log-likelihood for the salmon
  # example using a normal approximation
  alpha <- exp(pars[1])
  beta <- exp(pars[2])
  n0 <- exp(pars[3])
  sigmasq <- 0.2
  
  a1<-n0
  P1 <- 0
  
  H <- sigmasq
  
  l <-0
  for (i in 1:length(y)) { 
    Q <- a1
    a <- alpha*a1*exp(-beta*a1)
    G <- alpha*(1-beta*a1)*exp(-beta*a1)
    P <- G*P1*G + Q
    v <- y[i]-a
    FF<-(P + H)
    a1 <- a + P * (1/FF)
    P1 <- P - P * (1/FF) * P
    
    l <- l - 1/2 * ( log(FF ) + v*(1/FF)*v)
  }
  
  return(-l)
}

saltlik <- function(pars,y){
  # This function returns the negative log-likelihood for the salmon
  # example using a normal approximation with time dependent alpha
  n<-length(y)
  alpha <- exp(pars[1:n])
  beta <- exp(pars[n+1])
  n0 <- exp(pars[n+2])
  sigmasq <- 0.2
  
  a1<-n0
  P1 <- 0
  
  H <- sigmasq
  
  l <-0
  for (i in 1:length(y)) { 
    Q <- a1
    a <- alpha[i]*a1*exp(-beta*a1)
    G <- alpha[i]*(1-beta*a1)*exp(-beta*a1)
    P <- G*P1*G + Q
    v <- y[i]-a
    FF<-(P + H)
    a1 <- a + P * (1/FF)
    P1 <- P - P * (1/FF) * P
    
    l <- l - 1/2 * ( log(FF ) + v*(1/FF)*v)
  }
  
  return(-l)
}

#------------------------------------------------------------------
# Simulated Salmon data
#------------------------------------------------------------------
y <- c(104,217,226,466,764,781,903,1356,1189,1522,1334,1378,1146,1155,873,1918,1916,1445,1078,1573)
