#----------------------------------------------------------
# RBugs code for fitting eg2 - census data alone
#----------------------------------------------------------


# Population count data
y <- c(14,9,8,15,17,15,13,9,8,9,9,10,12,11,10,11,10,7,5,5,8,12,14,15,18,21)
T <- length(y)

# Winbugs model:
sink("census.bug")
cat("
    model	{
    # priors 
    phij ~ dunif(0,1)
    phia ~ dunif(0,1)
    im~dunif(0,5)
    rho ~ dunif(0,30)
    

    # Likelihood for population survey data

    # Inital values for numbers of first years, adults, immigrants and total
    for (k in 1:K){
       N1[k,1] ~ dnorm(100,0.0001)I(0,)   # First years  		
       NadSurv[k,1] ~ dnorm(100,0.0001)I(0,)  # Adults	 
       Nadimm[k,1] ~ dnorm(100,0.0001)I(0,) 	# Immigrant
       Ntot[k,1] <- NadSurv[k,1] + Nadimm[k,1] + N1[k,1]  # Total
    }
    
    # Numbers of first years, adults, immigrants and total for 2:T
    for (k in 1:K){
       for(t in 2:T){ 
          # System process
          mean1[k,t] <- 0.5*rho*phij*Ntot[k,t-1]
          N1[k,t] ~ dpois(mean1[k,t])
          mpo[k,t] <- Ntot[k,t-1]*im
          NadSurv[k,t] ~ dbin(phia,Ntot[k,t-1])
          Nadimm[k,t] ~ dpois(mpo[k,t])
          # Observation process
          Ntot[k,t] <- NadSurv[k,t] + Nadimm[k,t] + N1[k,t]
          y[k,t] ~ dpois(Ntot[k,t])
          }
    }     
    
    
    }  
    ",fill=TRUE)
sink()

K <- 1
y2 <- matrix(rep(y,K),nrow=K,ncol=T,byrow=T)
data.or <-list(T=T,y=y2,K=K)
N1 <- c(rep(50,T))
N12 <- matrix(rep(N1,K),nrow=K,ncol=T,byrow=T)
NadSurv <- y2/2
Nadimm <- y2/2
inits <- function(){list(phij=0.05,phia=0.6,rho=2.6,im=0.6,
                         N1=N12,NadSurv=NadSurv,Nadimm=Nadimm)}
parameters <- c("phij","phia","rho","im")
chain <- 1 
iter <- 20000
burn	<- 10000
thin <- 1

imm.out <- bugs(data=data.or,inits=inits,model.file = "census.bug",
                parameters=parameters, n.chains =chain, n.iter =iter,
                n.burnin=burn,n.thin=thin,debug=F, 
                bugs.directory =bugs.dir,clearWD=TRUE,codaPkg=TRUE)

#----------------------------------------------------------
# RBugs code for fitting eg2 - integrated census and CR
#----------------------------------------------------------
# Population count data
y <- c(14,9,8,15,17,15,13,9,8,9,9,10,12,11,10,11,10,7,5,5,8,12,14,15,18,21)
T <- length(y)
# Capture recapture data for females
mfem <- matrix(c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,
                 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,
                 0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,
                 0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,
                 0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,9,
                 0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,15,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,14,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,3,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,11,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,13,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,15,
                 1,2,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,
                 0,3,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,
                 0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,
                 0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,
                 0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,
                 0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
                 0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
                 0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,
                 0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
                 0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
                 0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,2,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,3,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,2,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,2,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,1,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,0,0,1,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,4,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,3,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3),nrow=50,ncol=26,byrow=T)


# Winbugs model:
sink("IPM_IMM.bug")
cat("
    model	{
    # priors 
    phij ~ dunif(0,1)
    phia ~ dunif(0,1)
    im~dunif(0,5)
    rho ~ dunif(0,30)
    p ~ dunif(0,1)
    
    # Likelihood for population survey data

    # Inital values for numbers of first years, adults, immigrants and total
    for (k in 1:K){
       N1[k,1] ~ dnorm(100,0.0001)I(0,)   # First years  		
       NadSurv[k,1] ~ dnorm(100,0.0001)I(0,)  # Adults	 
       Nadimm[k,1] ~ dnorm(100,0.0001)I(0,) 	# Immigrant
       Ntot[k,1] <- NadSurv[k,1] + Nadimm[k,1] + N1[k,1]  # Total
    }
    
    # Numbers of first years, adults, immigrants and total for 2:T
    for (k in 1:K){
       for(t in 2:T){ 
       # System process
       mean1[k,t] <- 0.5*rho*phij*Ntot[k,t-1]
       N1[k,t] ~ dpois(mean1[k,t])
       mpo[k,t] <- Ntot[k,t-1]*im
       NadSurv[k,t] ~ dbin(phia,Ntot[k,t-1])
       Nadimm[k,t] ~ dpois(mpo[k,t])
       # Observation process
       Ntot[k,t] <- NadSurv[k,t] + Nadimm[k,t] + N1[k,t]
       y[k,t] ~ dpois(Ntot[k,t])
       }
    }     
    
    # Likelihood for capture-recapture data
    # Follows a CJS 
    q <- 1-p
    for( i in 1:2*(T-1)) {
       m[i,1:T] ~ dmulti(pr[i,],r[i])
    } 	
    for(i in 1:2*(T-1)) {
       r[i] <- sum(m[i,])
    } 
    
    # m-array cell probabilities for juveniles
    for(i in 1:(T-1)) {
       pr[i,i]<-phij*p
       for(j in (i+1):(T-1)) {
          pr[i,j] <- phij*pow(phia,j-i)*pow(q,j-i)*p
       } 
    for( j in 1:(i-1)) {
       pr[i,j] <- 0
       }
       pr[i,T] <- 1-sum(pr[i,1:(T-1)])
    } 
    # m-array cell probabilities for adults
    for(i in 1:(T-1))   {
       pr[i+T-1,i] <- phia*p
       for(j in (i+1):(T-1))  {
          pr[i+T-1,j] <-pow(phia,j-i+1)*pow(q,j-i)*p 
       } 
       for( j in 1:(i-1))  {
          pr[i+T-1,j] <- 0
       } 
       pr[i+T-1,T] <- 1-sum(pr[i+T-1,1:(T-1)])
       } 
    }  
    ",fill=TRUE)
sink()

K <- 1
y2 <- matrix(rep(y,K),nrow=K,ncol=T,byrow=T)
mK <- mfem*K
data.or <-list(T=T,y=y2,K=K,m=mK)
N1 <- c(rep(50,T))
N12 <- matrix(rep(N1,K),nrow=K,ncol=T,byrow=T)
NadSurv <- y2/2
Nadimm <- y2/2
inits <- function(){list(phij=0.05,phia=0.6,rho=2.6,im=0.6,p=0.2,
                         N1=N12,NadSurv=NadSurv,Nadimm=Nadimm)}
parameters <- c("phij","phia","rho","im","p")
chain <- 1 
iter <- 20000
burn	<- 10000
thin <- 1

imm.out <- bugs(data=data.or,inits=inits,model.file = "IPM_IMM.bug",
                parameters=parameters, n.chains =chain, n.iter =iter,
                n.burnin=burn,n.thin=thin,debug=F, 
                bugs.directory =bugs.dir,clearWD=TRUE,codaPkg=TRUE)


